@@ -1,11 +1,15 @@
 ######################################################################
 # zwiki page parenting functionality
 
+import string
 from string import join
 from types import *
 from urllib import quote
 import Acquisition
 import Permissions
+import Globals
+from AccessControl import ClassSecurityInfo
+from Utils import flatten
 
 ###########################################################################
 # CLASS ZwikiParentsMixin
@@ -13,7 +17,7 @@ import Permissions
 # COLLABORATORS ZWikiPage
 ###########################################################################
 
-class ZWikiParentsMixin:
+class ParentsSupport:
     """
     This mix-in class encapsulates ZWikiPage's page parenting
     functionality.  I had to leave several related code fragments over
@@ -30,39 +34,54 @@ class ZWikiParentsMixin:
         {'id':'parents', 'type': 'lines', 'mode': 'w'},
         )
 
-    __ac_permissions__=(
-        (Permissions.Reparent, ('reparent',)),
-        )
+    security = ClassSecurityInfo()
 
     ######################################################################
     # METHOD CATEGORY: parenting
     ######################################################################
 
-    def reparent(self, parents=None, REQUEST=None):
-        """Reset parents property according to request."""
+    security.declareProtected(Permissions.Reparent, 'reparent')
+    def reparent(self, parents=None, REQUEST=None, pagename=None):
+        """
+        Reset parents property according to request.
+
+        parents can also be passed in the pagename option. This is for the
+        page management panel - it can't pass a parents argument because
+        of the parents attribute.
+        """
+        #XXX require at least a cookie like rename/delete ?
+        #not yet.. fix IssueNo0010 and maybe reparent is harmless enough
+        #if not (self._checkPermission(Permissions.Reparent, self) and
+        #        self.requestHasSomeId(REQUEST)):
+        #    raise 'Unauthorized', (
+        #        _('You are not authorized to reparent this ZWiki Page.'))
+                   
+        if pagename:
+            parents = pagename
         if parents is None:
             parents = REQUEST.get('parents', None)
         if parents is None:
             self.parents = []
         else:
             if type(parents) != ListType:
-                parents = [parents]
+                parents = string.split(parents)
             self.parents = parents
 
-        self._set_last_editor(REQUEST)
+        #self._setLastEditor(REQUEST)
 
         if REQUEST is not None:
             REQUEST.RESPONSE.redirect(REQUEST['URL1'])
 
+    security.declareProtected(Permissions.View, 'get_ancestors')
     def get_ancestors(self):
         """Return a trimmed nesting structure indicating this page's ancestors.
 
         See the WikiNesting docstring for nestings structure description."""
-        container = self.aq_parent
+        container = self.folder()
         ancestors = {}
         offspring = {}
         tops = {}                       # Ancestors that have no parents.
-        todo = {self.id(): None}
+        todo = {self.title_or_id(): None}
         while todo:
             doing = todo
             todo = {}
@@ -70,10 +89,15 @@ class ZWikiParentsMixin:
                 if ancestors.has_key(i):
                     continue            # We've already collected this one.
                 else:
+                    pagewiththisname = None
                     if not hasattr(container, i):
-                        continue        # Absent - ignore it.
+                        pagewiththisname = self.pageWithName(i)
+                        if not pagewiththisname:
+                            continue        # Absent - ignore it.
                     ancestors[i] = None
-                    obj = container[i]
+                    #cf IssueNo0108
+                    #obj = container[i]
+                    obj = pagewiththisname or getattr(container,i)
                     if not hasattr(obj, 'parents'):
                         continue
                     parents = obj.parents
@@ -96,20 +120,21 @@ class ZWikiParentsMixin:
             got.append(descend_ancestors(t, ancestors, did, offspring))
         return got
 
+    security.declareProtected(Permissions.View, 'context')
     def context(self, REQUEST=None, with_siblings=0, enlarge_current=0):
         """Return HTML showing this page's parents and siblings."""
-        myid = self.id()
+        myid = self.title_or_id()
         if with_siblings:
-            nesting = WikiNesting(self.aq_parent).get_up_and_back(myid)
+            nesting = WikiNesting(self.folder()).get_up_and_back(myid)
         else:
             nesting = self.get_ancestors()
             if (len(nesting) == 0  or
                 (len(nesting) == 1 and len(nesting[0]) == 1)) and not enlarge_current:
                 return "&nbsp;"
         #SKWM XXX
-        if self.aq_parent.title:
-            #titletxt = ' for %s' % (self.aq_parent.title)
-            titletxt = '%s ' % (self.aq_parent.title)
+        if self.folder().title:
+            #titletxt = ' for %s' % (self.folder().title)
+            titletxt = '%s ' % (self.folder().title)
         else:
             titletxt = ''
 
@@ -122,21 +147,22 @@ class ZWikiParentsMixin:
         else:
             suppress_hyperlink=0
 
-        #SKWM XXX
-        #detect when parents have gotten confused & reset if necessary
-        hierarchy = present_nesting(myid, nesting, self.wiki_url(),
+        hierarchy = self.present_nesting(myid, nesting, self.wiki_url(),
                                     enlarge_current=enlarge_current,
                                     suppress_hyperlink=suppress_hyperlink)
+        #SKWM XXX
+        #detect when parents have gotten confused & reset if necessary
         if hierarchy == '<ul>\n</ul>':
             self.parents = []
-            hierarchy = present_nesting(myid, nesting, self.wiki_url(),
+            hierarchy = self.present_nesting(myid, nesting, self.wiki_url(),
                                         enlarge_current=enlarge_current,
                                         suppress_hyperlink=suppress_hyperlink)
         
-        return '<small><ul><a href="%s/map#%s">%s contents</a>\n' % \
-               (self.page_url(), quote(self.id()), titletxt) + \
+        return '<small><ul><a href="%s/contents#%s" title="show wiki contents" accesskey="c">%s contents</a>\n' % \
+               (self.page_url(), quote(self.title_or_id()), titletxt) + \
                hierarchy + '\n</ul></small>'
 
+    security.declareProtected(Permissions.View, 'deepappend')
     def deepappend(self, nesting, pageid):
         """append a page to the very bottom of a nesting."""
         if type(nesting[-1]) is type([]):
@@ -149,34 +175,222 @@ class ZWikiParentsMixin:
                 nesting[-1] = [nesting[-1],pageid]
             return nesting
                 
+    security.declareProtected(Permissions.View, 'offspring')
     def offspring(self, REQUEST=None):
         """Return a presentation of all my offspring."""
-        myid = self.id()
-        nesting = WikiNesting(self.aq_parent)
-        return present_nesting(myid, nesting.get_offspring([myid]),
-                               self.wiki_url()) # SKWM
+        myid = self.title_or_id()
+        nesting = WikiNesting(self.folder())
+        return self.present_nesting(myid, nesting.get_offspring([myid]),
+                                    self.wiki_url()) # SKWM
     
-    def map(self, REQUEST=None):
+    security.declareProtected(Permissions.View, 'offspringAsList')
+    def offspringAsList(self, REQUEST=None):
+        """
+        Return all my offspring's page names as a flat list.
+        """
+        myid = self.title_or_id()
+        nesting = WikiNesting(self.folder())
+        list = flatten(nesting.get_offspring([myid]))
+        list.remove(myid)
+        return list
+
+    security.declareProtected(Permissions.View, 'offspringIdsAsList')
+    def offspringIdsAsList(self, REQUEST=None):
+        """
+        Return all my offspring's page ids as a flat list.
+        """
+        #return map(lambda x:self.pageWithNameOrId(x).id(),
+        #           self.offspringAsList())
+        # python 1.5 compatibility sucks, drop it soon ?
+        list = self.offspringAsList()
+        for i in range(len(list)):
+            list[i] = self.pageWithNameOrId(list[i]).id()
+        return list
+
+    security.declareProtected(Permissions.View, 'ancestorsAsList')
+    def ancestorsAsList(self, REQUEST=None):
+        """
+        Return the names of all my ancestor pages as a flat list, eldest first.
+
+        If there are multiple lines of ancestry, return only the first.
+        """
+        try:
+            return flatten(self.get_ancestors()[0])[:-1]
+        except:        # XXX temp
+            return []
+
+    security.declareProtected(Permissions.View, 'contents')
+    def contents(self, REQUEST=None):
         """Present the nesting layout of the entire wiki, showing:
 
         - All the independent nodes, ie those without parents or children, and
 
         - All the branches in the wiki - from the possibly multiple roots."""
         import string
-        map = WikiNesting(self.aq_parent).get_map()
+        map = WikiNesting(self.folder()).get_map()
         singletons = []
         combos = []
         rel = self.wiki_url()      # SKWM 
         for i in map:
             if type(i) == StringType:
-                singletons.append('<a href="%s/%s" name="%s">%s</a>' # SKWM add named targets
-                                  % (rel, quote(i), quote(i), i))
+                try:
+                    linktitle = self.folder()[i].linkTitle()
+                except:
+                    linktitle = ''
+                singletons.append(\
+                    '<a href="%s/%s" name="%s" title="%s">%s</a>'\
+                    % (rel, self.canonicalIdFrom(i), quote(i),
+                       linktitle,
+                       i))
             else:
                 combos.append(i)
-        return ("<h3>Singletons</h3> <ul> %s </ul><h3>The rest</h3> %s </p>"
-                % (join(singletons," "),
-                   present_nesting(self.id(), combos, rel)))
+        return self.contentspage(
+            self.present_nesting(self.title_or_id(), 
+                                 combos, 
+                                 rel),
+            singletons,
+            REQUEST=REQUEST)
 
+    map = contents # backwards compatibility
+
+    security.declareProtected(Permissions.View, 'present_nesting')
+    def present_nesting(self, myid, nesting, rel, did=None, _got=None, \
+                        indent="",enlarge_current=0,suppress_hyperlink=0):
+        # aaaah! dooo something...
+        """
+        Present an HTML outline of a nesting structure.
+
+        _got is used internally and should not be passed in except when
+        recursing.  See WikiNesting docstring for nesting structure
+        description.
+        This was a function, but I want to have more context available.
+        """
+        if did is None: did = []
+        if _got is None:
+            _got = ["<ul>"]
+            recursing = 0
+        else:
+            recursing = 1
+
+        for n in nesting:
+
+            if type(n) == ListType:
+                if n[0] == myid:
+                    # The entry is the current node - distinguish it.
+                    if enlarge_current:
+                        nodenm = '<big><big><big><big><strong>%s</strong></big></big></big></big>' % n[0]
+                        if suppress_hyperlink:
+                            _got.append('%s <li>%s' % (indent, nodenm))
+                        else:
+                            _got.append('%s <li><a href="%s/%s/backlinks" title="show backlinks for this page" accesskey="b">%s</a>'
+                                        % (indent,
+                                           rel,
+                                           self.canonicalIdFrom(n[0]),
+                                           nodenm))
+                    else:
+                        nodenm = "<strong>%s</strong>" % n[0]
+                        try:
+                            linktitle = self.folder()[n[0]].linkTitle()
+                        except:
+                            linktitle = ''
+                        _got.append(\
+                            '%s <li><a href="%s/%s" name="%s" title="%s">%s</a> <b><-- You are here.</b>'
+                            % (indent, rel, self.canonicalIdFrom(n[0]), quote(n[0]),
+                               linktitle,
+                               nodenm))
+                else:
+                    nodenm = n[0]
+                    try:
+                        linktitle = self.folder()[n[0]].linkTitle()
+                    except:
+                        linktitle = ''
+                    _got.append(\
+                        '%s <li><a href="%s/%s" name="%s" title="%s">%s</a>'
+                        % (indent, rel, self.canonicalIdFrom(n[0]), quote(n[0]),
+                           linktitle,
+                           nodenm))
+                if len(n) > 1:
+                    _got.append("<ul>")
+                    for i in n[1:]:
+                        if type(i) == ListType:
+                            _got = self.present_nesting(myid, [i], rel, did=did,
+                                                   _got=_got, indent=indent+'  ',
+                                                   enlarge_current=enlarge_current,
+                                                   suppress_hyperlink=suppress_hyperlink)
+                        else:
+                            if i == myid:
+                                # Distinguish that entry is for current node.
+                                if enlarge_current:
+                                    inm = '<big><big><big><big><strong>' + i + '</strong></big></big></big></big>'
+                                    if suppress_hyperlink:
+                                        _got.append('%s <li>%s' % (indent, inm))
+                                    else:
+                                        _got.append('%s <li><a href="%s/%s/backlinks" title="show backlinks for this page" accesskey="b">%s</a>' % (indent, rel, self.canonicalIdFrom(i), inm))
+                                else:
+                                    inm = "<strong>" + i + "</strong>"
+                                    try:
+                                        linktitle = self.folder()[i].linkTitle()
+                                    except:
+                                        linktitle = ''
+                                    _got.append(\
+                                        '%s <li><a href="%s/%s" name="%s" title="%s">%s</a> <b><-- You are here.</b>'
+                                        % (indent, rel, self.canonicalIdFrom(i), quote(i),
+                                           linktitle,
+                                           inm))
+                            else:
+                                inm = i
+                                try:
+                                    linktitle = self.folder()[i].linkTitle()
+                                except:
+                                    linktitle = ''
+                                _got.append(\
+                                    '%s <li><a href="%s/%s" name="%s" title="%s">%s</a>'
+                                    % (indent, rel, self.canonicalIdFrom(i), quote(i),
+                                       linktitle,
+                                       inm))
+                    _got.append("</ul>")
+                else:
+                    # Parents whose children were omitted - indicate:
+                    _got[-1] = _got[-1] + " ..."
+            else:
+                if n == myid:
+                    # The entry is for the current node - distinguish.
+                    if enlarge_current:
+                        inm = '<big><big><big><big><strong>' + n + '</strong></big></big></big></big>'
+                        if suppress_hyperlink:
+                            _got.append('%s <li>%s' % (indent, inm))
+                        else:
+                            _got.append('%s <li><a href="%s/%s/backlinks" title="show backlinks for this page" accesskey="b">%s</a>' % (indent, rel, self.canonicalIdFrom(n), inm))
+                    else:
+                        inm = "<strong>" + n + "</strong>"
+                        try:
+                            linktitle = self.folder()[n].linkTitle()
+                        except:
+                            linktitle = ''
+                        _got.append(\
+                            '%s <li><a href="%s/%s" name="%s" title="%s">%s</a> <b><-- You are here.</b>'
+                            % (indent, rel, self.canonicalIdFrom(n), quote(n),
+                               linktitle,
+                               inm))
+                else:
+                    inm = n
+                    try:
+                        linktitle = self.folder()[n].linkTitle()
+                    except:
+                        linktitle = ''
+                    _got.append(\
+                        '%s <li><a href="%s/%s" name="%s" title="%s">%s</a>'
+                        % (indent, rel, self.canonicalIdFrom(n), quote(n),
+                           linktitle,
+                           inm))
+
+        if recursing:
+            return _got
+        else:
+            _got.append("</ul>")
+            return join(_got, "\n")
+
+Globals.InitializeClass(ParentsSupport) # install permissions
     
 ###########################################################################
 # CLASS WikiNesting
@@ -217,7 +431,7 @@ class WikiNesting(Acquisition.Implicit):
           - .childmap:  {'node1': ['child1', ...], ...}
           - .roots: {'root1': None, ...}"""
 
-        pages = self.container.objectValues(spec='ZWiki Page')
+        pages = self.container.objectValues(spec='ZWiki Page') # XXX optimize
         pagenames = []
         parentmap = {}                  # 'node': ['parent1', ...]
         childmap = {}                   # 'node': ['child1', ...]
@@ -226,7 +440,7 @@ class WikiNesting(Acquisition.Implicit):
             parents = pg.parents
             if type(parents) != ListType: # SKWM
                 parents = []
-            pgnm = pg.id()
+            pgnm = pg.title_or_id()
             pagenames.append(pgnm)
 
             # Parents is direct:
@@ -325,92 +539,6 @@ class WikiNesting(Acquisition.Implicit):
 # FUNCTION CATEGORY: parenting helper functions
 ######################################################################
 
-def present_nesting(myid, nesting, rel, did=None, _got=None, indent="", \
-                    enlarge_current=0, suppress_hyperlink=0):
-    # aaaah! dooo something...
-    """
-    Present an HTML outline of a nesting structure.
-
-    _got is used internally and should not be passed in except when recursing.
-
-    See WikiNesting docstring for nesting structure description.
-    """
-    if did is None: did = []
-    if _got is None:
-        _got = ["<ul>"]
-        recursing = 0
-    else:
-        recursing = 1
-
-    for n in nesting:
-
-        if type(n) == ListType:
-            if n[0] == myid:
-                # The entry is the current node - distinguish it.
-                if enlarge_current:
-                    nodenm = '<big><big><big><big><strong>%s</strong></big></big></big></big>' % n[0]
-                    if suppress_hyperlink:
-                        _got.append('%s <li>%s' % (indent, nodenm))
-                    else:
-                        _got.append('%s <li><a href="%s/%s/backlinks">%s</a>'
-                                    % (indent, rel, quote(n[0]), nodenm))
-                else:
-                    nodenm = "<strong>%s</strong>" % n[0]
-                    _got.append('%s <li><a href="%s/%s" name="%s">%s</a>'
-                                % (indent, rel, quote(n[0]), quote(n[0]), nodenm))
-            else:
-                nodenm = n[0]
-                _got.append('%s <li><a href="%s/%s" name="%s">%s</a>'
-                            % (indent, rel, quote(n[0]), quote(n[0]), nodenm))
-            if len(n) > 1:
-                _got.append("<ul>")
-                for i in n[1:]:
-                    if type(i) == ListType:
-                        _got = present_nesting(myid, [i], rel, did=did,
-                                               _got=_got, indent=indent+'  ',
-                                               enlarge_current=enlarge_current,
-                                               suppress_hyperlink=suppress_hyperlink)
-                    else:
-                        if i == myid:
-                            # Distinguish that entry is for current node.
-                            if enlarge_current:
-                                inm = '<big><big><big><big><strong>' + i + '</strong></big></big></big></big>'
-                                if suppress_hyperlink:
-                                    _got.append('%s <li>%s' % (indent, inm))
-                                else:
-                                    _got.append('%s <li><a href="%s/%s/backlinks">%s</a>' % (indent, rel, quote(i), inm))
-                            else:
-                                inm = "<strong>" + i + "</strong>"
-                                _got.append('%s <li><a href="%s/%s" name="%s">%s</a>' % (indent, rel, quote(i), quote(i), inm))
-                        else:
-                            inm = i
-                            _got.append('%s <li><a href="%s/%s" name="%s">%s</a>' % (indent, rel, quote(i), quote(i), inm))
-                _got.append("</ul>")
-            else:
-                # Parents whose children were omitted - indicate:
-                _got[-1] = _got[-1] + " ..."
-        else:
-            if n == myid:
-                # The entry is for the current node - distinguish.
-                if enlarge_current:
-                    inm = '<big><big><big><big><strong>' + n + '</strong></big></big></big></big>'
-                    if suppress_hyperlink:
-                        _got.append('%s <li>%s' % (indent, inm))
-                    else:
-                        _got.append('%s <li><a href="%s/%s/backlinks">%s</a>' % (indent, rel, quote(n), inm))
-                else:
-                    inm = "<strong>" + n + "</strong>"
-                    _got.append('%s <li><a href="%s/%s" name="%s">%s</a>' % (indent, rel, quote(n), quote(n), inm))
-            else:
-                inm = n
-                _got.append('%s <li><a href="%s/%s" name="%s">%s</a>' % (indent, rel, quote(n), quote(n), inm))
-
-    if recursing:
-        return _got
-    else:
-        _got.append("</ul>")
-        return join(_got, "\n")
-
 def descend_ancestors(page, ancestors, did, children):
     """Create nesting of ancestors leading to page.
 
